SuperStrict

Rem
	bbdoc: Exceptions
	about: This module provides a collection of Exception classes.
Endrem
Module siliziumbrain.exceptions
ModuleInfo "Version: v1.01"
ModuleInfo "Author: M0rgenstern - Siliziumbrain - Kevin Peter Hoffeld"
ModuleInfo "License: Public Domain"
ModuleInfo "History v1.00: 12 July 2012 - Final Release"
ModuleInfo "Histroy v1.01: 13 July 2012 - Now a child class of TObject."

Import brl.system
Import siliziumbrain.uberobject

Rem
	bbdoc: (Abstract) Exception Class.
	about: Every Exception inherits from this class.
endrem
Type TException Extends TObject Abstract
	Field _sOwnType:String
	Field _sOwnMessage:String
	Field _sGivenMessage:String
	Field _sTimeWhenThrown:String
	Field _iID:Int
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Getter for the exception type as string.
	endrem
	Method __OwnType:String()
		Return _sOwnType
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Setter for the exception Type as string.
	endrem
	Method __SetOwnType(psOwnType:String)
		_sOwnType = psOwnType
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Getter for the intern exception message.
	endrem
	Method __OwnMessage:String()
		Return _sOwnMessage
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Setter for the intern exception message.
	endrem
	Method __SetOwnMessage(psOwnMessage:String)
		_sOwnMessage = psOwnMessage
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Getter for the extern exception message.
	endrem
	Method __GivenMessage:String()
		Return _sGivenMessage
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Setter for the extern exception message.
	endrem
	Method __SetGivenMessage(psGivenMessage:String)
		_sGivenMessage = psGivenMessage
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Getter for the time when the exception was thrown as a string.
	endrem
	Method __TimeWhenThrown:String()
		Return _sTimeWhenThrown
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Setter for the time when the exception was thrown as a string.
	endrem
	Method __SetTimeWhenThrown(psTimeWhenThrown:String)
		_sTimeWhenThrown = psTimeWhenThrown
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Getter for the ID of the exception.
	endrem
	Method __ID:Int()
		Return _iID
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Setter for the ID of the exception.
	endrem
	Method __SetID(piID:Int)
		_iID = piID
	End Method
	
	Rem
		bbdoc: The Exception as a string.
		about: This will return a nicely formatted string which descripes the exception.
		returns: String with all needed information about the exception.
	endrem
	Method ToString:String()
		Return ("An Exception of the Type '" + __OwnType() + "' has occurred at " + ..
				__TimeWhenThrown() + ". ~n ~t -- Itself says: " + __OwnMessage() + " ~n ~t " + ..
				"-- The extern input says: " + ..
				__GivenMessage() + " ~n ~t -- Exception ID: " + __ID() + ".")
	End Method
	
	Rem
		bbdoc:Checks if two objects are equal.
	End Rem
	Method Equals:Byte(ptoOtherObject:TObject)
		If(ptoOtherObject = Null) Then Return False
		
		If(Not TException(ptoOtherObject)) Then Return False
		
		Local obj:TException = TException(ptoOtherObject)
		
		If(obj.__OwnType() = __OwnType()) Then
			If(obj.__OwnMessage() = __OwnMessage()) Then
				If(obj.__GivenMessage() = __GivenMessage()) Then
					Return True
				EndIf
			EndIf
		EndIf
		
		Return False
	End Method
	
	Method Copy:TException() Abstract

End Type

Rem
	bbdoc: Nullpointer Exception Class.
	about: This Exception is used when e.g. a function gets 'null' as a paremter although it should not.
endrem
Type TNullPointerException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TNullPointerException.NewException.
	endrem
	Method NewException:TNullPointerException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("Tried to access to an object which is null (not initialized).")
		__SetOwnType("NullPointerException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TNullPointerException()
		Local copEx:TNullPointerException = New TNullPointerException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type

Rem
	bbdoc: IndexOutOfBounds Exception Class.
	about: This Exception is used when e.g. a array is indexed with a too large index.
endrem
Type TIndexOutOfBoundsException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TIndexOutOfBoundsException.NewException.
	endrem
	Method NewException:TIndexOutOfBoundsException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("Tried to access to an array with a too big or to small index.")
		__SetOwnType("IndexOutOfBoundsException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TIndexOutOfBoundsException()
		Local copEx:TIndexOutOfBoundsException = New TIndexOutOfBoundsException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type

Rem
	bbdoc: ArgumentOutOfRangeException Exception Class.
	about: This Exception is used when e.g. a method gets an argument which is too big or too small.
endrem
Type TArgumentOutOfRangeException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TArgumentOutOfRangeException.NewException.
	endrem
	Method NewException:TArgumentOutOfRangeException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("Tried to assign an argument which is out of the permitted range.")
		__SetOwnType("ArgumentOutOfRangeException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TArgumentOutOfRangeException()
		Local copEx:TArgumentOutOfRangeException = New TArgumentOutOfRangeException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type

Rem
	bbdoc: IllegalArgumentException Exception Class.
	about: This Exception is used when e.g. a method gets an argument which is just invalid for this method.
endrem
Type TIllegalArgumentException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TIllegalArgumentException.NewException.
	endrem
	Method NewException:TIllegalArgumentException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("Assigned argument is illegal.")
		__SetOwnType("IllegalArgumentException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TIllegalArgumentException()
		Local copEx:TIllegalArgumentException = New TIllegalArgumentException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type

Rem
	bbdoc: InvalidOperationException Exception Class.
	about: This Exception is used when e.g. a object tries to call a method although the object is not in the right state to call that method (e.g. empty list calls Remove).
endrem
Type TInvalidOperationException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TInvalidOperationException.NewException.
	endrem
	Method NewException:TInvalidOperationException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("The state of the object forbids to use the method (in this way).")
		__SetOwnType("InvalidOperationException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TInvalidOperationException()
		Local copEx:TInvalidOperationException = New TInvalidOperationException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type