SuperStrict

Rem
	bbdoc: Queue.
	about: This module provides a Queue class (#TQueue). It can do everything what a queue should can do.
Endrem
Module siliziumbrain.queue
ModuleInfo "Version: v1.00"
ModuleInfo "Author: M0rgenstern - Siliziumbrain - Kevin Peter Hoffeld"
ModuleInfo "License: Public Domain"
ModuleInfo "History v1.00: 12 July 2012 - Final Release"

Import brl.linkedlist
Import siliziumbrain.exceptions
Import siliziumbrain.uberobject

Rem
	bbdoc: TQueue class.
	about: A queue which can add object at the and and remove objects at the beginning.
endrem
Type TQueue Extends TObject
	Global IDs:Int = 1
	
	Field _tlContent:TList
	Field _sTimeWhenCreated:String
	Field _iID:Int
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Returns the intern list.
	endrem
	Method __Content:TList()
		Return _tlContent
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Sets the intern list.
	endrem
	Method __SetContent(ptlContent:TList)
		_tlContent = ptlContent
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Returns the time when the queue was created as a string.
	endrem
	Method __TimeWhenCreated:String()
		Return _sTimeWhenCreated
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Sets the time when the queue was created as a string.
	endrem
	Method __SetTimeWhenCreated(psTimeWhenCreated:String)
		_sTimeWhenCreated = psTimeWhenCreated
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Returns the ID of the queue.
	endrem
	Method __ID:Int()
		Return _iID
	End Method
	
	Rem
		bbdoc: Private Method. Do not use.
		about: Sets the ID of the queue.
	endrem
	Method __SetID(piID:Int)
		_iID = piID
	End Method
	
	Rem
		bbdoc: Returns the amount of objects in the queue.
		about: Sets the time when the list was created as a string.
		returns: Amount of elements in the queue.
	endrem
	Method Count:Int()
		Return __Content().Count()
	End Method
	
	Rem
		bbdoc: Enqueue an object.
		about: Adds @poObjectToEnqueue at the end of the queue.
	endrem
	Method Enqueue(poObjectToEnqueue:Object)
		__Content().AddLast(poObjectToEnqueue)
	End Method
	
	Rem
		bbdoc: Deques an object.
		about: Returns the first object in the queue and removes it from the queue. <br>
		%Throws: #TInvalidOperationException if the queue is empty. <br>
		returns: First object in queue.
	endrem
	Method Dequeue:Object()
		If(Count() > 0) Then
			Local oDequObj:Object = __Content().First()
			__Content().RemoveFirst()
			Return oDequObj
		EndIf

		Throw New TInvalidOperationException.NewException("In 'TQueue.(m)Dequeue()' the queue " + ToString() + " has no elements.")
	End Method
	
	Rem
		bbdoc: Peeks an object.
		about: Returns the first object in the queue without removing it from the queue. <br>
		%Throws: #TInvalidOperationException if the queue is empty. <br>
		returns: First object in queue.
	endrem
	Method Peek:Object()
		If(Count() > 0) Then
			Return __Content().First()
		EndIf
		
		Throw New TInvalidOperationException.NewException("In 'TQueue.(m)Peek()' the queue " + ToString() + " has no elements.")
	End Method
	
	Rem
		bbdoc: Checks if a certain object is in the queue.
		about: Checks if @poObject is in the queue. <br>
		returns: %True if object is in queue, otherwise %false.
	endrem
	Method Contains:Byte(poObject:Object)
		Return __Content().Contains(poObject)
	End Method
	
	Rem
		bbdoc: Clears the queue.
		about: Removes every object from the queue.
	endrem
	Method Clear()
		__Content().Clear()
	End Method
	
	Rem
		bbdoc: Constructor.
		about: Returns a new queue.
	endrem
	Method New()
		__SetID(IDs)
		__SetTimeWhenCreated(CurrentDate() + " - " + CurrentTime())
		__SetContent(New TList)
		
		IDs:+1
	End Method
	
	Rem
		bbdoc: Returns the queue as a string.
		about: Gives certain information about the queue. <br>
		returns: The queue as a string.
	endrem
	Method ToString:String()
		Return ("Queue: " + __ID() + ". Created: " + __TimeWhenCreated() + ". ~n ~t ~t ~t Has intern name: " + Super.ToString())
	End Method
	
	Rem
		bbdoc: Checks if the queue is equal to another object.
		about: Checks if the queue equals @ptoOtherObject. <br>
		returns: %true if they are equal, otherwise %false.
	endrem
	Method Equals:Byte(ptoOtherObject:TObject)
		If(ptoOtherObject = Null) Then Return False
		
		If(Not TQueue(ptoOtherObject)) Then Return False
		
		Local obj:TQueue = TQueue(ptoOtherObject)
		
		If(Count() = obj.Count()) Then
			For Local index:Int = 0 Until Count()
				If(obj.__Content().ValueAtIndex(index) <> __Content().ValueAtIndex(index)) Then
					Return False
				EndIf
				Return True
			Next
		EndIf
		
		Return False
	End Method
	
	Rem
		bbdoc: Returns the queue as an array.
		about: Every entry of the queue is copied (flat) into an array. <br>
		returns: Queue as an array.
	endrem
	Method ToArray:Object[] ()
		Local oaArray:Object[]
		oaArray = New Object[Count()]
		Local index:Int = 0
		
		For Local o:Object = EachIn __Content()
			oaArray[index] = o
			index:+1
		Next
		
		Return oaArray
	End Method
	
	Rem
		bbdoc: Copies the queue to an array (flat).
		about: The queue is copied into the array @oaArrayToCopyTo, starting at index @iStartIndex. <br>
		The array is given to the method as pointer. So nothing is returned. <br>
		%Throws: #TNullPointerException if @oaArrayToCopyTo is %null. <br>
		%Throws: #TArgumentOutOfRangeException if @iStartIndex is < 0. <br>
		%Throws: #TIllegalArgumentException if @iStartIndex is too big. <br>
	endrem
	Method CopyTo(oaArrayToCopyTo:Object[] Var, iStartIndex:Int = 0)
		If(oaArrayToCopyTo = Null) Then
			Throw New TNullPointerException.NewException("In 'TQueue.(m)CopyTo(oaArrayToCopyTo, iStartIndex)' " + ..
				"~n ~t ~t the parameter 'oaArrayToCopyTo' is 'null'. ~n ~t ~t Queue which calls this method: " + ToString())
		EndIf
		If(iStartIndex < 0) Then
			Throw New TArgumentOutOfRangeException.NewException("In 'TQueue.(m)CopyTo(oaArrayToCopyTo, iStartIndex)' " + ..
				"~n ~t ~t the parameter 'iStartIndex' is smaller than '0'. It is: " + iStartIndex + ". " + ..
				"~n ~t ~t Queue which calls this method: " + ToString() + ".")
		EndIf
		If(iStartIndex >= (oaArrayToCopyTo.Length - 1)) Then
			Throw New TIllegalArgumentException.NewException("In 'TQueue.(m)CopyTo(oaArrayToCopyTo, iStartIndex)' " + ..
				"~n ~t ~t the parameter 'iStartIndex' is too big. It is " + iStartIndex + ..
				". It could be max: " + (oaArrayToCopyTo.Length - 1) + ..
				". ~n ~t ~t Queue which calls this method: " + ToString() + ".")
		EndIf
		Local iMaxIndex:Int = oaArrayToCopyTo.Length - Count()
		If(iStartIndex > iMaxIndex) Then
			Throw New TIllegalArgumentException.NewException("In 'TQueue.(m)CopyTo(oaArrayToCopyTo, iStartIndex)' " + ..
				"~n ~t ~t the parameter 'iStartIndex' is too big. ~n ~t ~t It is " + iStartIndex + ..
				". But should be max (for " + Count() + " objects in queue):" + iMaxIndex + ". " + ..
				"~n ~t ~t Queue which calls this method: " + ToString() + ".")
		EndIf
		
		For Local o:Object = EachIn __Content()
			oaArrayToCopyTo[iStartIndex] = o
			iStartIndex:+1
		Next
	End Method
	
	Rem
		bbdoc:Copies the queue (deep).
		about: Creates a deep copy of the queue (as far as possible). <br>
		returns: New queue which is a (deep) copy of this queue.
	End Rem
	Method Copy:TQueue()
		Local newQueue:TQueue = New TQueue
		
		For Local obj:Object = EachIn __Content()
			If(TObject(obj)) Then
				newQueue.Enqueue(TObject(obj).Copy())
			Else
				newQueue.Enqueue(obj)
			EndIf
		Next
		
		Return newQueue
	End Method
End Type